# -*- coding: utf-8 -*-

# Copyright (C) 2013-2014 Avencall
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

import string
import random

from sqlalchemy.sql import and_
from sqlalchemy.exc import  IntegrityError
from xivo_dao.alchemy.linefeatures import LineFeatures as LineSchema
from xivo_dao.alchemy.usersip import UserSIP as UserSIPSchema, UserSIP
from xivo_dao.alchemy.useriax import UserIAX as UserIAXSchema, UserIAX
from xivo_dao.alchemy.usercustom import UserCustom as UserCustomSchema, \
    UserCustom
from xivo_dao.alchemy.sccpline import SCCPLine as SCCPLineSchema, SCCPLine
from xivo_dao.alchemy.user_line import UserLine as UserLineSchema
from xivo_dao.alchemy.extension import Extension
from xivo_dao.helpers.db_utils import commit_or_abort
from xivo_dao.helpers.db_manager import daosession
from xivo_dao.data_handler import errors
from xivo_dao.data_handler.exception import DataError
from model import LineSIP, LineIAX, LineSCCP, LineCUSTOM
from xivo_dao.data_handler.line.model import LineOrdering


DEFAULT_ORDER = [LineOrdering.name, LineOrdering.context]


@daosession
def get(session, line_id):
    line = session.query(LineSchema).filter(LineSchema.id == line_id).first()

    if not line:
        raise errors.not_found('Line', id=line_id)

    return _convert_to_model(line)


@daosession
def get_by_user_id(session, user_id):
    line = (
        session.query(LineSchema)
        .join(UserLineSchema, and_(UserLineSchema.user_id == user_id,
                                   UserLineSchema.line_id == LineSchema.id,
                                   UserLineSchema.main_line == True,
                                   UserLineSchema.main_user == True))
        .first())

    if not line:
        raise errors.not_found('Line', user_id=user_id)

    return _convert_to_model(line)


@daosession
def get_by_number_context(session, number, context):
    line = (
        session.query(LineSchema)
        .join(Extension, and_(Extension.exten == number,
                              Extension.context == context))
        .join(UserLineSchema, and_(UserLineSchema.extension_id == Extension.id,
                                   LineSchema.id == UserLineSchema.line_id,
                                   UserLineSchema.main_line == True,
                                   UserLineSchema.main_user == True))
    ).first()

    if not line:
        raise errors.not_found('Line', number=number, context=context)

    return _convert_to_model(line)


@daosession
def find_all(session, order=None):
    line_rows = _new_query(session, order).all()

    if not line_rows:
        return []

    lines = []
    for line_row in line_rows:
        protocol = line_row.protocol.lower()

        if protocol == 'sip':
            line_protocol = LineSIP.from_data_source(line_row)
        elif protocol == 'iax':
            line_protocol = LineIAX.from_data_source(line_row)
        elif protocol == 'sccp':
            line_protocol = LineSCCP.from_data_source(line_row)
        elif protocol == 'custom':
            line_protocol = LineCUSTOM.from_data_source(line_row)

        lines.append(line_protocol)

    return lines


@daosession
def find_all_by_protocol(session, protocol, order=None):
    line_rows = (_new_query(session, order)
                 .filter(LineSchema.protocol == protocol.lower())
                 .all())

    return _rows_to_line_model(line_rows)


@daosession
def find_all_by_name(session, name, order=None):
    search = '%%%s%%' % name.lower()

    line_rows = (_new_query(session, order)
                 .filter(LineSchema.name.ilike(search))
                 .all())

    return _rows_to_line_model(line_rows)


@daosession
def find_all_by_device_id(session, device_id, order=None):
    line_rows = (_new_query(session, order)
                 .filter(LineSchema.device == str(device_id))
                 .all())

    return _rows_to_line_model(line_rows)


@daosession
def find_by_user_id(session, user_id, main_line=True, main_user=True):
    line_row = (_new_query(session)
                .join(UserLineSchema,
                      and_(UserLineSchema.user_id == user_id,
                           UserLineSchema.line_id == LineSchema.id,
                           UserLineSchema.main_line == main_line,
                           UserLineSchema.main_user == main_user))
                .first())

    if line_row:
        return _convert_to_model(line_row)
    return None


def _rows_to_line_model(line_rows):
    if not line_rows:
        return []

    lines = []
    for line_row in line_rows:
        lines.append(_convert_to_model(line_row))

    return lines


def _convert_to_model(line):
    protocol = line.protocol.lower()
    protocol_row = _get_protocol_row(line)

    if protocol == 'sip':
        line_protocol = LineSIP.from_data_source(line)
        line_protocol.update_from_data_source(protocol_row)
    elif protocol == 'iax':
        line_protocol = LineIAX.from_data_source(line)
        line_protocol.update_from_data_source(protocol_row)
    elif protocol == 'sccp':
        line_protocol = LineSCCP.from_data_source(line)
        line_protocol.update_from_data_source(protocol_row)
    elif protocol == 'custom':
        line_protocol = LineCUSTOM.from_data_source(line)
        line_protocol.update_from_data_source(protocol_row)
        line_protocol.name = protocol_row.interface

    return line_protocol


@daosession
def _get_protocol_row(session, line):
    protocol = line.protocol.lower()
    if protocol == 'sip':
        row = session.query(UserSIP).filter(line.protocolid == UserSIP.id).first()
    elif protocol == 'iax':
        row = session.query(UserIAX).filter(line.protocolid == UserIAX.id).first()
    elif protocol == 'sccp':
        row = session.query(SCCPLine).filter(line.protocolid == SCCPLine.id).first()
    elif protocol == 'custom':
        row = session.query(UserCustom).filter(line.protocolid == UserCustom.id).first()

    if not row:
        raise errors.not_found('Line%s' % protocol.upper(), protocolid=line.protocolid)

    return row


@daosession
def reset_device(session, device_id):
    session.query(LineSchema).filter(LineSchema.device == str(device_id)).update({'device': ''})


@daosession
def provisioning_id_exists(session, provd_id):
    line = session.query(LineSchema.id).filter(LineSchema.provisioningid == provd_id).count()
    if line > 0:
        return True
    return False


@daosession
def create(session, line):
    derived_line = _build_derived_line(session, line)
    _commit_or_abort(session, DataError.on_create, derived_line)

    line_row = _build_line_row(line, derived_line)
    _commit_or_abort(session, DataError.on_create, line_row)

    line.id = line_row.id
    return line


def _commit_or_abort(session, error, *elements):
    with commit_or_abort(session, error, 'Line'):
        for element in elements:
            session.add(element)


@daosession
def edit(session, line):
    derived_line = _fetch_derived_line(session, line)
    if derived_line is None:
        session.rollback()
        raise errors.not_found('LineProtocol', protocolid=line.protocolid)
    line.update_data_source(derived_line)

    line_row = _fetch_line(session, line.id)
    if line_row is None:
        session.rollback()
        raise errors.not_found('Line', id=line.id)
    line.update_data_source(line_row)

    _commit_or_abort(session, DataError.on_edit, derived_line, line_row)


@daosession
def update_xivo_userid(session, line, main_user):
    if line.protocol.lower() == 'sip':
        sip_line = _fetch_derived_line(session, line)
        sip_line.setvar = 'XIVO_USERID=%s' % main_user.id

        _commit_or_abort(session, DataError.on_edit, sip_line)


@daosession
def delete_user_references(session, line_id):
    line = _fetch_line(session, line_id)
    if line.protocol.lower() == 'sip':
        sip_line = _fetch_derived_line(session, line)
        sip_line.callerid = None
        sip_line.setvar = ''

        _commit_or_abort(session, DataError.on_edit, sip_line)


def _fetch_line(session, line_id):
    return session.query(LineSchema).get(line_id)


def _fetch_derived_line(session, line):
    protocol = line.protocol.lower()

    if protocol == 'sip':
        return session.query(UserSIPSchema).get(line.protocolid)
    elif protocol == 'iax':
        return session.query(UserIAXSchema).get(line.protocolid)
    elif protocol == 'sccp':
        return session.query(SCCPLineSchema).get(line.protocolid)
    elif protocol == 'custom':
        return session.query(UserCustomSchema).get(line.protocolid)


def _build_derived_line(session, line):
    protocol = line.protocol.lower()

    if protocol == 'sip':
        derived_line = _build_sip_line(session, line)
    elif protocol == 'iax':
        derived_line = _build_iax_line(line)
    elif protocol == 'sccp':
        derived_line = _build_sccp_line(line)
    elif protocol == 'custom':
        derived_line = _build_custom_line(line)

    if hasattr(derived_line, 'id'):
        del derived_line.id

    return derived_line


def _build_line_row(line, derived_line):
    line.protocolid = derived_line.id
    line_row = line.to_data_source(LineSchema)
    if line_row.configregistrar is None:
        line_row.configregistrar = 'default'
    return line_row


def _build_sip_line(session, line):
    if getattr(line, 'username', None) is None:
        line.username = generate_random_hash(session, UserSIPSchema.name)

    if getattr(line, 'secret', None) is None:
        line.secret = generate_random_hash(session, UserSIPSchema.secret)

    line.name = line.username
    line_row = line.to_data_source(UserSIPSchema)

    line_row.name = line.username
    line_row.username = ''
    line_row.type = 'friend'
    if line_row.category is None:
        line_row.category = 'user'  # enum: (user,trunk)

    return line_row


def generate_random_hash(session, column):
    token = random_hash()
    query = session.query(column)

    count = query.filter(column == token).count()
    while count > 0:
        token = random_hash()
        count = query.filter(column == token).count()

    return token


def random_hash(length=8):
    pool = string.lowercase + string.digits
    return ''.join(random.choice(pool) for _ in range(length))


def _build_iax_line(line):
    raise NotImplementedError


def _build_sccp_line(line):
    raise NotImplementedError


def _build_custom_line(line):
    raise NotImplementedError


@daosession
def delete(session, line):
    try:
        with commit_or_abort(session, DataError.on_delete, 'Line'):
            nb_row_affected = session.query(LineSchema).filter(LineSchema.id == line.id).delete()
            _delete_line(session, line)
    except IntegrityError:
        session.rollback()
        raise errors.resource_associated('Line', 'Resource', line_id=line.id)

    if nb_row_affected == 0:
        errors.not_found('Line', line.id)

    return nb_row_affected


def _delete_line(session, line):
    protocolid = int(line.protocolid)
    protocol = line.protocol.lower()
    if protocol == 'sip':
        _delete_sip_line(session, protocolid)
    elif protocol == 'iax':
        _delete_iax_line(session, protocolid)
    elif protocol == 'sccp':
        _delete_sccp_line(session, protocolid)
    elif protocol == 'custom':
        _delete_custom_line(session, protocolid)


def _delete_sip_line(session, protocolid):
    session.query(UserSIPSchema).filter(UserSIPSchema.id == protocolid).delete()


def _delete_iax_line(session, protocolid):
    session.query(UserIAXSchema).filter(UserIAXSchema.id == protocolid).delete()


def _delete_sccp_line(session, protocolid):
    session.query(SCCPLineSchema).filter(SCCPLineSchema.id == protocolid).delete()


def _delete_custom_line(session, protocolid):
    session.query(UserCustomSchema).filter(UserCustomSchema.id == protocolid).delete()


def _new_query(session, order=None):
    order = order or DEFAULT_ORDER
    return session.query(LineSchema).order_by(*order)


@daosession
def associate_extension(session, extension, line_id):
    line_row = (session.query(LineSchema)
                .filter(LineSchema.id == line_id)
                .first())

    if line_row:
        line_row.number = extension.exten
        line_row.context = extension.context

        _commit_or_abort(session, DataError.on_edit, line_row)


@daosession
def dissociate_extension(session, extension):
    line_row = (session.query(LineSchema)
                .filter(LineSchema.number == extension.exten)
                .filter(LineSchema.context == extension.context)
                .first())

    if line_row:
        line_row.number = ''
        _commit_or_abort(session, DataError.on_edit, line_row)
